import os
from typing import Optional

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.database import DatabaseSettings
from cpl_core.database.context import DatabaseContext
from cpl_core.dependency_injection import ServiceProviderABC, ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironment
from cpl_discord import get_discord_collection
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from dotenv import load_dotenv

from ui_tests.test_on_ready_event import TestOnReadyEvent


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)

        self._config: Optional[ConfigurationABC] = None

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        load_dotenv()
        configuration.add_environment_variables('KDB_TEST_')
        configuration.add_environment_variables('DISCORD_')

        cwd = os.path.dirname(os.path.realpath(__file__))
        configuration.add_json_file(f'{cwd}/config/appsettings.json', optional=False)
        configuration.add_json_file(f'{cwd}/config/appsettings.{environment.host_name}.json', optional=True)

        self._config = configuration
        return configuration

    def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_logging()
        services.add_translation()
        db_settings: DatabaseSettings = self._config.get_configuration(DatabaseSettings)
        db_settings_with_pw = DatabaseSettings()
        pw = self._config.get_configuration('DB_PASSWORD')
        db_settings_with_pw.from_dict({
            "Host": db_settings.host,
            "User": db_settings.user,
            "Password": '' if pw is None else pw,
            "Database": db_settings.database,
            "Charset": db_settings.charset,
            "UseUnicode": db_settings.use_unicode,
            "Buffered": db_settings.buffered,
            "AuthPlugin": db_settings.auth_plugin
        })
        services.add_db_context(DatabaseContext, db_settings_with_pw)

        services.add_discord()
        dc = get_discord_collection(services)
        dc.add_event(DiscordEventTypesEnum.on_ready.value, TestOnReadyEvent)

        return services.build_service_provider()
