import os
import unittest

from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console, ForegroundColorEnum
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnReadyABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC


class TestOnReadyEvent(OnReadyABC):

    def __init__(
            self,
            config: ConfigurationABC,
            bot: DiscordBotServiceABC,
            services: ServiceProviderABC,
            client_utils: ClientUtilsServiceABC,
            t: TranslatePipe
    ):
        OnReadyABC.__init__(self)

        self._config = config
        self._bot = bot
        self._services = services
        self._client_utils = client_utils
        self._t = t

    async def on_ready(self):
        if self._config.get_configuration('IS_UNITTEST'):
            return

        Console.write_line('\nStarting tests:\n')
        loader = unittest.TestLoader()
        path = f'{os.path.dirname(os.path.realpath(__file__))}/../'
        tests = loader.discover(path, pattern='*_test_case.py')
        runner = unittest.TextTestRunner()
        runner.run(tests)
        # for cls in CommandTestABC.__subclasses__():
        #     service: CommandTestABC = self._services.get_service(cls)
        #     await service.run(self._tests)

        await self._bot.close()
