from datetime import datetime
import discord

import bot
from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from ui_tests.src.ui_tests_shared.command_selectors_enum import CommandSelectorsEnum
from ui_tests_shared.command_test_case_with_app import CommandTestCaseWithApp
from ui_tests_shared.decorators import Async


class InfoCommandTestCase(CommandTestCaseWithApp):

    def get_embed(self) -> discord.Embed:
        client_utils: ClientUtilsServiceABC = self._services.get_service(ClientUtilsServiceABC)
        client = client_utils.get_client(self._bot.user.id, self._test_settings.guild_id)
        embed = discord.Embed(
            title=self._t.transform('modules.base.info.title'),
            description=self._t.transform('modules.base.info.description'),
            color=int('ef9d0d', 16)
        )

        embed.add_field(name=self._t.transform('modules.base.info.fields.version'), value=bot.__version__)
        start_time = self._config.get_configuration('Bot_StartTime')
        ontime = round((datetime.now() - datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S.%f')).total_seconds() / 3600, 2)
        embed.add_field(name=self._t.transform('modules.base.info.fields.ontime'), value=f'{ontime}h')
        embed.add_field(name=self._t.transform('modules.base.info.fields.sent_message_count'), value=client.sent_message_count, inline=False)
        embed.add_field(name=self._t.transform('modules.base.info.fields.received_message_count'), value=client.received_message_count)
        embed.add_field(name=self._t.transform('modules.base.info.fields.deleted_message_count'), value=client.deleted_message_count, inline=False)
        embed.add_field(name=self._t.transform('modules.base.info.fields.received_command_count'), value=client.received_command_count)
        embed.add_field(name=self._t.transform('modules.base.info.fields.moved_users_count'), value=client.moved_users_count)
        from bot.module_list import ModuleList
        modules = ModuleList.get_modules()
        modules = modules.select(lambda x: x.__name__.replace('Module', ''))
        embed.add_field(name=self._t.transform('modules.base.info.fields.modules'), value='\n'.join(modules), inline=False)
        return embed

    @Async.async_func
    async def test_info(self):
        correct_response = self.get_embed()
        self.send_command('info', CommandSelectorsEnum.info)

        def check(m: discord.Message):
            return m.embeds[0] == correct_response and m.author.id == self._test_settings.bot_id

        response: discord.Message = await self._bot.wait_for('message', check=check)
        self.assertEqual(len(response.embeds), 1)

        self.assertEqual(response.embeds[0], correct_response)
        await self._bot.close()
